#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Ideas on City Bike Challenge
# 
# Based on the data available at [Divvy Data](https://www.divvybikes.com/system-data), how to guess where people usually live and where the usually work? This notebook suggests some directions and shows some results.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')


# ## The data
# 
# [Divvy Data](https://www.divvybikes.com/system-data) publishes a sample of the data. 

# In[3]:


from pyensae.datasource import download_data
file = download_data("Divvy_Trips_2016_Q3Q4.zip", url="https://s3.amazonaws.com/divvy-data/tripdata/")


# We know the stations.

# In[4]:


import pandas
stations = df = pandas.read_csv("Divvy_Stations_2016_Q3.csv")
df.head()


# And we know the trips.

# In[5]:


bikes = df = pandas.read_csv("Divvy_Trips_2016_Q3.csv")
df.head()


# ## First assumption
# 
# Which time do you go to work? How do you go to work? Maybe by bicycles, maybe in the morning, everyday... Let's see the distribution of the stop time in every station of one particular day of the week.

# In[6]:


import pandas
from datetime import datetime, time
df["dtstart"] = pandas.to_datetime(df.starttime, infer_datetime_format=True)
df["dtstop"] = pandas.to_datetime(df.stoptime, infer_datetime_format=True)
df["stopday"] = df.dtstop.apply(lambda r: datetime(r.year, r.month, r.day))
df["stoptime"] = df.dtstop.apply(lambda r: time(r.hour, r.minute, 0))
df["stoptime10"] = df.dtstop.apply(lambda r: time(r.hour, (r.minute // 10)*10, 0))  # every 10 minutes


# In[7]:


df['stopweekday'] = df['dtstop'].dt.dayofweek


# ### Big stations
# 
# We average the number of the number of bicycles which stops at the station.

# In[8]:


key = ["to_station_name", "to_station_id", "stopday"]
perday = df[key + ["trip_id"]].groupby(key, as_index=False).count()
ave = perday.groupby(key[:-1], as_index=False).mean()
ave.columns = "to_station_name", "to_station_id", "mean stops per day"
ave.head()


# In[9]:


ax = ave["mean stops per day"].hist(bins=50, figsize=(14,4))
ax.set_xlim([0, 300])
ax.set_yscale("log")
ax.set_title("mean stops per day")
ax.set_xlabel("mean stops per day")
ax.set_ylabel("number of stations");


# In[10]:


ave[ave["mean stops per day"] > 20].shape, ave.shape


# In[11]:


df.dtstop.min(), df.dtstop.max()


# For about half of the stations, more than 20 bicycles stops between July and October of 2016. Let's take one of them.

# In[12]:


ave[ave["mean stops per day"] > 20].head()


# In[13]:


key = ["to_station_name", "to_station_id", "stopweekday", "stoptime10"]
snippet = df[key + ["trip_id"]].groupby(key, as_index=False).count()
snippet.columns = "to_station_name", "to_station_id", "stopweekday", "stoptime10", "nb_stops"
snippet21 = snippet[snippet["to_station_id"] == 21].copy()
snippet21.head()


# In[14]:


snippet21.tail()


# In[15]:


from ensae_projects.datainc.data_bikes import add_missing_time
import matplotlib.pyplot as plt
fig, ax = plt.subplots(1,1, figsize=(14,4))
full_snippet21 = add_missing_time(snippet21, "stoptime10", delay=10, values="nb_stops")
full_snippet21["rstops"] = full_snippet21.nb_stops.rolling(7).mean()
full_snippet21.plot(x="stoptime10", y="rstops", figsize=(14,4), kind="area", ax=ax)
ax.set_title("station 21");


# In[16]:


from ensae_projects.datainc.data_bikes import add_missing_time
import matplotlib.pyplot as plt
fig, ax = plt.subplots(1,1, figsize=(14,4))
sni = snippet[snippet["to_station_id"] == 341].copy()
sni = add_missing_time(sni, "stoptime10", delay=10, values="nb_stops")
sni["rstops"] = sni.nb_stops.rolling(7).mean()
sni.plot(x="stoptime10", y="rstops", figsize=(14,4), kind="area", ax=ax)
ax.set_title("station 341");


# ### Indicator
# 
# If a station is inside working area, people will arrive in the morning and will leave in the evening. If it is a living area, people should leave in the morning and arrive in the evening. Let's note $L_t$ the number of bicyles leaving a station. Let's compute the ratio:
# 
# $$R = \frac{\sum_{t=8am}^{12am} L_t}{\sum_{t=0am}^{12pm} L_t}$$

# In[17]:


col = full_snippet21["stoptime10"]
R21 = full_snippet21.nb_stops[(col > time(8,0,0)) & (col < time(12,0,0))].sum() /  \
      full_snippet21.nb_stops[col >= time(0,0,0)].sum()
R21


# In[18]:


key = ["to_station_name", "to_station_id", "stopweekday", "stoptime10"]
agg = bikes[key + ["trip_id"]].groupby(key, as_index=False).count().copy()

ratios = {}
for ids in set(stations.id):
    sni = agg[agg["to_station_id"] == ids]
    sni.columns = "to_station_name", "to_station_id", "stopweekday", "stoptime10", "nb_stops"
    num = sni.nb_stops[(sni["stoptime10"] >= time(8,0,0)) & (sni["stoptime10"] <= time(12,0,0))].sum()
    den = sni.nb_stops[sni["stoptime10"] >= time(0,0,0)].sum()
    if den > 0:
        ratios[ids] = num * 1.0 / den


# In[19]:


import numpy
stations_ratio = stations.copy()
stations_ratio["ratio"] = stations.id.apply(lambda x: ratios.get(x, numpy.nan))
stations_ratio.head()


# In[20]:


stations_ratio["ratio"].hist(bins=50);


# We choose the median.

# In[21]:


stations_ratio.ratio.median()


# And we draw a map.

# In[22]:


from ensae_projects.datainc.data_bikes import folium_html_stations_map
xy = []
for els in stations_ratio.apply(lambda row: (row["latitude"], row["longitude"], row["ratio"], row["name"]), axis=1):
    name = "%s %1.2f" % (els[3], els[2])
    color = "red" if els[2] >= 0.1648655139289145 else "blue"
    xy.append( ( (els[0], els[1]), (name, color)))
folium_html_stations_map(xy, width="80%")


# If we assume that people have bigger flats than working space, we should assume there are more living areas than working areas.

# In[23]:


stations_ratio.ratio.quantile(0.6)


# In[24]:


from ensae_projects.datainc.data_bikes import folium_html_stations_map
xy = []
for els in stations_ratio.apply(lambda row: (row["latitude"], row["longitude"], row["ratio"], row["name"]), axis=1):
    name = "%s %1.2f" % (els[3], els[2])
    color = "red" if els[2] >= 0.18241042345276873 else "blue"
    xy.append( ( (els[0], els[1]), (name, color)))
folium_html_stations_map(xy, width="80%")


# In[25]: